import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

public
class SocketClient extends Frame implements ActionListener {
    
    private Insets              mInsets = new Insets(3,3,3,3);

    private ClientTitlePanel    tp;
    private ClientConnectPanel  cp;
    private ClientVerifyPanel   vp;
    private ClientStatusPanel   sp;

    private String              serverName;
    private int                 portNo;
    private int                 failedCount = 1;

    private HostPortMessage     serverhpm;
    private ClientSecuredComm   csc;

    private String              browser = "/usr/npac/communicator-v405/netscape ";

    public
    SocketClient() { }

    protected void
    createGui() {
        setLayout(new BorderLayout());

        tp = new ClientTitlePanel("NPAC Client");
        add(tp, "North");

        cp = new ClientConnectPanel(this);
        cp.setTexts(serverhpm.host, serverhpm.port);

        vp = new ClientVerifyPanel(this);

        sp = new ClientStatusPanel(this);
        add(sp, "South");
    }

    // connect to the Coordinator 
    // to obtain the server host name and port number to be connected

    public boolean connectToCoordinator(String host, int port) {
        try {
            Socket socket = new Socket(host, port);
            ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
            serverhpm = (HostPortMessage)ois.readObject();
            ois.close();
            socket.close();
            return true;
        } catch (Exception ex) {
            System.err.println("Client Exception-> " + ex);
        }
        return false;
    }

    // connect to assigned server

    public boolean connectToServer(String host, int port) {
        try {
            Socket socket = new Socket(host, port);
            csc = new ClientSecuredComm(socket);

            if (!csc.init()) {
                sp.setText("Key verification failed");
                return false;
            } else {
                sp.setText("Key verification successful");
            }
            return true;
        } catch (Exception ex) {
            System.err.println("Client Exception-> " + ex);
        }
        return false;
    }

    protected void
    changePanel(boolean b) {
        if (b) {
            if (isAncestorOf(vp)) remove(vp);
            add(cp, "Center");
        } else {
            if (isAncestorOf(cp)) remove(cp);
            add(vp, "Center");
        }
    }

   /**
    *  Handle action events from various controls
    *  Overrides Component.action().
    */
    public
    void actionPerformed(ActionEvent pe) {
        // User selected month from the choice
        String actionLabel = pe.getActionCommand();

        if (actionLabel.equals("Connect")) {

            String host = cp.getHost();
            int    port = cp.getPort();

            sp.setText("Connecting to <" + host + ":" + port + ">");

            if (!connectToServer(host, port)) return;

            sp.setText("Connected to <" + host + ":" + port + ">");
            changePanel(false);

        } else if (actionLabel.equals("Verify")) {

            String userid = vp.getUsername();
            String passwd = vp.getPassword();
            if (userid.length() == 0) {
                sp.setText("Null account!  Try again!");
                return;
            }
            if (passwd.length() == 0) {
                sp.setText("Null password!  Try again!");
                return;
            }
            sp.setText("Connecting to NPAC Server");

            // Set user information and send it to server
            csc.setupMessage(userid, passwd);
            csc.send();

            if (csc.receive()) {
                WFMessage wfm = (WFMessage)csc.getMessage();
                if (wfm.type == 1) {
                    sp.setText("Authentication successful");
                    java.lang.Runtime rt = java.lang.Runtime.getRuntime();
                    try {
                        rt.exec(browser + wfm.msg);
                    } catch (Exception ex) {
                        System.err.println("Exception-> " + ex);
                    }
                    csc.done();
                    System.exit(0);
                } else {
                    sp.setText("Error: <" + wfm.type + ">" + wfm.msg);
                    System.err.println("Username/Password verification failed");
                    vp.clearTexts();
                    cp.resetTexts();
                    changePanel(true);
                }
                return;
            }
        } else if (actionLabel.equals("Cancel")) {
            cp.resetTexts();
            vp.clearTexts();
            changePanel(true);
        }
        validate();
    }

    /**
     *  Paint nice raized 3D background.
     *  Overrides Component.paint().
     */
    public void
    paint(Graphics pg)
    {
        pg.setColor(Color.black);
        pg.drawLine(1, 0, getBounds().width-2, 0);
        pg.drawLine(0, 1, 0, getBounds().height-2);
        pg.drawLine(1, getBounds().height-1, getBounds().width - 2, getBounds().height - 1);
        pg.drawLine(getBounds().width - 1, 1, getBounds().width - 1, getBounds().height - 2);

        pg.setColor(getBackground());
        //pg.draw3DRect(1, 1, getBounds().width - 3, getBounds().height - 3, true);
        pg.fill3DRect(1, 1, getBounds().width - 2, getBounds().height - 2, true);
    }

    /**
     *  For better looks, let Layout Manager leave some margin.
     */
	public
	Insets
	getInsets()
	{
	    return mInsets;
	}

    public static void
    main(String[] args) {

        if (args.length != 4) {
            System.err.println("java SocketClient -CHost hostname -CPort #");
            System.exit(1);
        }

        // parse the input arguments

        int i = 0;
        while (!args[i++].equals("-CHost")) {};
        String host = args[i];

        i = 0;
        while (!args[i++].equals("-CPort")) {};
        int port = Integer.parseInt(args[i]);

        System.out.println("Connecting to " + host + ":" + port);

        // initialization

        SocketClient sc = new SocketClient();

        if (!sc.connectToCoordinator(host, port)) {
            System.err.println("Connection to " + host + ":" + port + " failed");
            System.exit(1);
        }
        System.out.println("Connected to " + host + ":" + port);

        sc.setSize(400, 300);
        sc.createGui();
        sc.changePanel(true);
        sc.show();
    }
}
